@@ -23,13 +23,13 @@ STRIP_KEYS = ['cmake', 'reply', 'backtrace', 'backtraceGraph', 'version']
 
 class CMakeFileAPI:
     def __init__(self, build_dir: Path):
-        self.build_dir            = build_dir
-        self.api_base_dir         = self.build_dir / '.cmake' / 'api' / 'v1'
-        self.request_dir          = self.api_base_dir / 'query' / 'client-meson'
-        self.reply_dir            = self.api_base_dir / 'reply'
-        self.cmake_sources        = []   # type: T.List[CMakeBuildFile]
+        self.build_dir = build_dir
+        self.api_base_dir = self.build_dir / '.cmake' / 'api' / 'v1'
+        self.request_dir = self.api_base_dir / 'query' / 'client-meson'
+        self.reply_dir = self.api_base_dir / 'reply'
+        self.cmake_sources = []   # type: T.List[CMakeBuildFile]
         self.cmake_configurations = []   # type: T.List[CMakeConfiguration]
-        self.kind_resolver_map    = {
+        self.kind_resolver_map = {
             'codemodel': self._parse_codemodel,
             'cmakeFiles': self._parse_cmakeFiles,
         }
@@ -51,7 +51,7 @@ class CMakeFileAPI:
         }
 
         query_file = self.request_dir / 'query.json'
-        query_file.write_text(json.dumps(query, indent=2))
+        query_file.write_text(json.dumps(query, indent=2), encoding='utf-8')
 
     def load_reply(self) -> None:
         if not self.reply_dir.is_dir():
@@ -75,20 +75,20 @@ class CMakeFileAPI:
         # Debug output
         debug_json = self.build_dir / '..' / 'fileAPI.json'
         debug_json = debug_json.resolve()
-        debug_json.write_text(json.dumps(index, indent=2))
+        debug_json.write_text(json.dumps(index, indent=2), encoding='utf-8')
         mlog.cmd_ci_include(debug_json.as_posix())
 
         # parse the JSON
         for i in index['objects']:
-            assert(isinstance(i, dict))
-            assert('kind' in i)
-            assert(i['kind'] in self.kind_resolver_map)
+            assert isinstance(i, dict)
+            assert 'kind' in i
+            assert i['kind'] in self.kind_resolver_map
 
             self.kind_resolver_map[i['kind']](i)
 
     def _parse_codemodel(self, data: T.Dict[str, T.Any]) -> None:
-        assert('configurations' in data)
-        assert('paths' in data)
+        assert 'configurations' in data
+        assert 'paths' in data
 
         source_dir = data['paths']['source']
         build_dir = data['paths']['build']
@@ -103,7 +103,7 @@ class CMakeFileAPI:
             src_dir = Path(dir_entry.get('source', '.'))
             bld_dir = Path(dir_entry.get('build', '.'))
             src_dir = src_dir if src_dir.is_absolute() else source_dir / src_dir
-            bld_dir = bld_dir if bld_dir.is_absolute() else build_dir  / bld_dir
+            bld_dir = bld_dir if bld_dir.is_absolute() else build_dir / bld_dir
             src_dir = src_dir.resolve()
             bld_dir = bld_dir.resolve()
 
@@ -313,7 +313,7 @@ class CMakeFileAPI:
         if not real_path.exists():
             raise CMakeException(f'File "{real_path}" does not exist')
 
-        data = json.loads(real_path.read_text())
+        data = json.loads(real_path.read_text(encoding='utf-8'))
         assert isinstance(data, dict)
         for i in data.keys():
             assert isinstance(i, str)
