@@ -11,22 +11,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import re
+import dataclasses
 import functools
 import typing as T
 from pathlib import Path
 
 from .. import mlog
 from .. import mesonlib
-from ..environment import Environment
 
 from .base import DependencyException, SystemDependency
 from .pkgconfig import PkgConfigDependency
 from .misc import threads_factory
 
 if T.TYPE_CHECKING:
-    from ..environment import Properties
+    from ..environment import Environment, Properties
 
 # On windows 3 directory layouts are supported:
 # * The default layout (versioned) installed:
@@ -83,6 +84,10 @@ if T.TYPE_CHECKING:
 #     2.4 Ensure that all libraries have the same boost tag (and are thus compatible)
 #   3. Select the libraries matching the requested modules
 
+@dataclasses.dataclass(eq=False, order=False)
+class UnknownFileException(Exception):
+    path: Path
+
 @functools.total_ordering
 class BoostIncludeDir():
     def __init__(self, path: Path, version_int: int):
@@ -152,7 +157,7 @@ class BoostLibraryFile():
         elif self.nvsuffix in ['a', 'lib']:
             self.static = True
         else:
-            raise DependencyException(f'Unable to process library extension "{self.nvsuffix}" ({self.path})')
+            raise UnknownFileException(self.path)
 
         # boost_.lib is the dll import library
         if self.basename.startswith('boost_') and self.nvsuffix == 'lib':
@@ -232,15 +237,15 @@ class BoostLibraryFile():
         return abitag
 
     def is_boost(self) -> bool:
-        return any([self.name.startswith(x) for x in ['libboost_', 'boost_']])
+        return any(self.name.startswith(x) for x in ['libboost_', 'boost_'])
 
     def is_python_lib(self) -> bool:
-        return any([self.mod_name.startswith(x) for x in BoostLibraryFile.boost_python_libs])
+        return any(self.mod_name.startswith(x) for x in BoostLibraryFile.boost_python_libs)
 
     def fix_python_name(self, tags: T.List[str]) -> T.List[str]:
         # Handle the boost_python naming madeness.
         # See https://github.com/mesonbuild/meson/issues/4788 for some distro
-        # specific naming variantions.
+        # specific naming variations.
         other_tags = []  # type: T.List[str]
 
         # Split the current modname into the base name and the version
@@ -320,7 +325,7 @@ class BoostLibraryFile():
         elif vscrt in ['/MTd', '-MTd']:
             return (self.runtime_static or not self.static) and self.runtime_debug
 
-        mlog.warning(f'Boost: unknow vscrt tag {vscrt}. This may cause the compilation to fail. Please consider reporting this as a bug.', once=True)
+        mlog.warning(f'Boost: unknown vscrt tag {vscrt}. This may cause the compilation to fail. Please consider reporting this as a bug.', once=True)
         return True
 
     def get_compiler_args(self) -> T.List[str]:
@@ -382,7 +387,7 @@ class BoostDependency(SystemDependency):
         # Finally, look for paths from .pc files and from searching the filesystem
         self.detect_roots()
 
-    def check_and_set_roots(self, roots: T.List[Path]) -> None:
+    def check_and_set_roots(self, roots: T.List[Path], use_system: bool) -> None:
         roots = list(mesonlib.OrderedSet(roots))
         for j in roots:
             #   1. Look for the boost headers (boost/version.hpp)
@@ -394,7 +399,7 @@ class BoostDependency(SystemDependency):
             if not inc_dirs:
                 continue
 
-            lib_dirs = self.detect_lib_dirs(j)
+            lib_dirs = self.detect_lib_dirs(j, use_system)
             self.is_found = self.run_check(inc_dirs, lib_dirs)
             if self.is_found:
                 self.boost_root = j
@@ -405,7 +410,7 @@ class BoostDependency(SystemDependency):
 
         The machine file values are defaulted to the environment values.
         """
-        # XXX: if we had a TypedDict we woudn't need this
+        # XXX: if we had a TypedDict we wouldn't need this
         incdir = props.get('boost_includedir')
         assert incdir is None or isinstance(incdir, str)
         libdir = props.get('boost_librarydir')
@@ -429,14 +434,14 @@ class BoostDependency(SystemDependency):
 
         rootdir = props.get('boost_root')
         # It shouldn't be possible to get here without something in boost_root
-        assert(rootdir)
+        assert rootdir
 
         raw_paths = mesonlib.stringlistify(rootdir)
         paths = [Path(x) for x in raw_paths]
-        if paths and any([not x.is_absolute() for x in paths]):
+        if paths and any(not x.is_absolute() for x in paths):
             raise DependencyException('boost_root path given in machine file must be absolute')
 
-        self.check_and_set_roots(paths)
+        self.check_and_set_roots(paths, use_system=False)
 
     def run_check(self, inc_dirs: T.List[BoostIncludeDir], lib_dirs: T.List[Path]) -> bool:
         mlog.debug('  - potential library dirs: {}'.format([x.as_posix() for x in lib_dirs]))
@@ -487,8 +492,8 @@ class BoostDependency(SystemDependency):
                 comp_args += c_args
                 link_args += l_args
 
-            comp_args = list(set(comp_args))
-            link_args = list(set(link_args))
+            comp_args = list(mesonlib.OrderedSet(comp_args))
+            link_args = list(mesonlib.OrderedSet(link_args))
 
             self.modules_found = [x.mod_name for x in selected_modules]
             self.modules_found = [x[6:] for x in self.modules_found]
@@ -532,17 +537,19 @@ class BoostDependency(SystemDependency):
         candidates = [x for x in candidates if x.exists()]
         return [self._include_dir_from_version_header(x) for x in candidates]
 
-    def detect_lib_dirs(self, root: Path) -> T.List[Path]:
+    def detect_lib_dirs(self, root: Path, use_system: bool) -> T.List[Path]:
         # First check the system include paths. Only consider those within the
         # given root path
-        system_dirs_t = self.clib_compiler.get_library_dirs(self.env)
-        system_dirs = [Path(x) for x in system_dirs_t]
-        system_dirs = [x.resolve() for x in system_dirs if x.exists()]
-        system_dirs = [x for x in system_dirs if mesonlib.path_is_in_root(x, root)]
-        system_dirs = list(mesonlib.OrderedSet(system_dirs))
 
-        if system_dirs:
-            return system_dirs
+        if use_system:
+            system_dirs_t = self.clib_compiler.get_library_dirs(self.env)
+            system_dirs = [Path(x) for x in system_dirs_t]
+            system_dirs = [x.resolve() for x in system_dirs if x.exists()]
+            system_dirs = [x for x in system_dirs if mesonlib.path_is_in_root(x, root)]
+            system_dirs = list(mesonlib.OrderedSet(system_dirs))
+
+            if system_dirs:
+                return system_dirs
 
         # No system include paths were found --> fall back to manually looking
         # for library dirs in root
@@ -567,13 +574,13 @@ class BoostDependency(SystemDependency):
         arch_list_64 = ['64']
 
         raw_list = dirs + subdirs
-        no_arch = [x for x in raw_list if not any([y in x.name for y in arch_list_32 + arch_list_64])]
+        no_arch = [x for x in raw_list if not any(y in x.name for y in arch_list_32 + arch_list_64)]
 
         matching_arch = []  # type: T.List[Path]
         if '32' in self.arch:
-            matching_arch = [x for x in raw_list if any([y in x.name for y in arch_list_32])]
+            matching_arch = [x for x in raw_list if any(y in x.name for y in arch_list_32)]
         elif '64' in self.arch:
-            matching_arch = [x for x in raw_list if any([y in x.name for y in arch_list_64])]
+            matching_arch = [x for x in raw_list if any(y in x.name for y in arch_list_64)]
 
         return sorted(matching_arch) + sorted(no_arch)
 
@@ -615,14 +622,22 @@ class BoostDependency(SystemDependency):
         return libs
 
     def detect_libraries(self, libdir: Path) -> T.List[BoostLibraryFile]:
-        libs = []  # type: T.List[BoostLibraryFile]
+        libs = set()  # type: T.Set[BoostLibraryFile]
         for i in libdir.iterdir():
-            if not i.is_file() or i.is_symlink():
+            if not i.is_file():
                 continue
-            if not any([i.name.startswith(x) for x in ['libboost_', 'boost_']]):
+            if not any(i.name.startswith(x) for x in ['libboost_', 'boost_']):
+                continue
+            # Windows binaries from SourceForge ship with PDB files alongside
+            # DLLs (#8325).  Ignore them.
+            if i.name.endswith('.pdb'):
                 continue
 
-            libs += [BoostLibraryFile(i)]
+            try:
+                libs.add(BoostLibraryFile(i.resolve()))
+            except UnknownFileException as e:
+                mlog.warning('Boost: ignoring unknown file {} under lib directory'.format(e.path.name))
+
         return [x for x in libs if x.is_boost()]  # Filter out no boost libraries
 
     def detect_split_root(self, inc_dir: Path, lib_dir: Path) -> None:
@@ -645,7 +660,7 @@ class BoostDependency(SystemDependency):
         try:
             boost_pc = PkgConfigDependency('boost', self.env, {'required': False})
             if boost_pc.found():
-                boost_root = boost_pc.get_pkgconfig_variable('prefix', {'default': None})
+                boost_root = boost_pc.get_pkgconfig_variable('prefix', [], None)
                 if boost_root:
                     roots += [Path(boost_root)]
         except DependencyException:
@@ -679,11 +694,6 @@ class BoostDependency(SystemDependency):
         else:
             tmp = []  # type: T.List[Path]
 
-            # Homebrew
-            brew_boost = Path('/usr/local/Cellar/boost')
-            if brew_boost.is_dir():
-                tmp += [x for x in brew_boost.iterdir()]
-
             # Add some default system paths
             tmp += [Path('/opt/local')]
             tmp += [Path('/usr/local/opt/boost')]
@@ -695,7 +705,7 @@ class BoostDependency(SystemDependency):
             tmp = [x.resolve() for x in tmp]
             roots += tmp
 
-        self.check_and_set_roots(roots)
+        self.check_and_set_roots(roots, use_system=True)
 
     def log_details(self) -> str:
         res = ''
