@@ -12,6 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import os
 import shutil
@@ -23,17 +24,19 @@ from .. import build
 from .. import coredata
 from .. import mlog
 from ..dependencies import find_external_dependency, Dependency, ExternalLibrary
-from ..mesonlib import MesonException, File, FileOrString, version_compare, Popen_safe
+from ..mesonlib import MesonException, File, version_compare, Popen_safe
 from ..interpreter import extract_required_kwarg
-from ..interpreter.type_checking import NoneType
+from ..interpreter.type_checking import INSTALL_DIR_KW, INSTALL_KW, NoneType
 from ..interpreterbase import ContainerTypeInfo, FeatureDeprecated, KwargInfo, noPosargs, FeatureNew, typed_kwargs
-from ..programs import ExternalProgram, NonExistingExternalProgram
+from ..programs import NonExistingExternalProgram
 
 if T.TYPE_CHECKING:
     from . import ModuleState
     from ..dependencies.qt import QtPkgConfigDependency, QmakeQtDependency
     from ..interpreter import Interpreter
     from ..interpreter import kwargs
+    from ..mesonlib import FileOrString
+    from ..programs import ExternalProgram
 
     QtDependencyType = T.Union[QtPkgConfigDependency, QmakeQtDependency]
 
@@ -103,6 +106,8 @@ class QtBaseModule(ExtensionModule):
     def __init__(self, interpreter: 'Interpreter', qt_version: int = 5):
         ExtensionModule.__init__(self, interpreter)
         self.qt_version = qt_version
+        # It is important that this list does not change order as the order of
+        # the returned ExternalPrograms will change as well
         self.tools: T.Dict[str, ExternalProgram] = {
             'moc': NonExistingExternalProgram('moc'),
             'uic': NonExistingExternalProgram('uic'),
@@ -120,8 +125,6 @@ class QtBaseModule(ExtensionModule):
 
     def compilers_detect(self, state: 'ModuleState', qt_dep: 'QtDependencyType') -> None:
         """Detect Qt (4 or 5) moc, uic, rcc in the specified bindir or in PATH"""
-        # It is important that this list does not change order as the order of
-        # the returned ExternalPrograms will change as well
         wanted = f'== {qt_dep.version}'
 
         def gen_bins() -> T.Generator[T.Tuple[str, str], None, None]:
@@ -151,11 +154,11 @@ class QtBaseModule(ExtensionModule):
             # Ensure that the version of qt and each tool are the same
             def get_version(p: ExternalProgram) -> str:
                 _, out, err = Popen_safe(p.get_command() + arg)
-                if b.startswith('lrelease') or not qt_dep.version.startswith('4'):
+                if name == 'lrelease' or not qt_dep.version.startswith('4'):
                     care = out
                 else:
                     care = err
-                return care.split(' ')[-1].replace(')', '').strip()
+                return care.rsplit(' ', maxsplit=1)[-1].replace(')', '').strip()
 
             p = state.find_program(b, required=False,
                                    version_func=get_version,
@@ -336,6 +339,7 @@ class QtBaseModule(ExtensionModule):
                 name,
                 state.subdir,
                 state.subproject,
+                state.environment,
                 self.tools['rcc'].get_command() + ['-name', name, '-o', '@OUTPUT@'] + extra_args + ['@INPUT@'] + DEPFILE_ARGS,
                 sources,
                 [f'{name}.cpp'],
@@ -355,6 +359,7 @@ class QtBaseModule(ExtensionModule):
                     name,
                     state.subdir,
                     state.subproject,
+                    state.environment,
                     self.tools['rcc'].get_command() + ['-name', '@BASENAME@', '-o', '@OUTPUT@'] + extra_args + ['@INPUT@'] + DEPFILE_ARGS,
                     [rcc_file],
                     [f'{name}.cpp'],
@@ -529,8 +534,8 @@ class QtBaseModule(ExtensionModule):
     @typed_kwargs(
         'qt.compile_translations',
         KwargInfo('build_by_default', bool, default=False),
-        KwargInfo('install', bool, default=False),
-        KwargInfo('install_dir', (str, NoneType)),
+        INSTALL_KW,
+        INSTALL_DIR_KW,
         KwargInfo('method', str, default='auto'),
         KwargInfo('qresource', (str, NoneType), since='0.56.0'),
         KwargInfo('rcc_extra_arguments', ContainerTypeInfo(list, str), listify=True, default=[], since='0.56.0'),
@@ -584,6 +589,7 @@ class QtBaseModule(ExtensionModule):
                 f'qt{self.qt_version}-compile-{ts}',
                 outdir,
                 state.subproject,
+                state.environment,
                 cmd,
                 [ts],
                 ['@BASENAME@.qm'],
